package cz.jzitnik.chronos.controllers;

import cz.jzitnik.chronos.entities.Interaction;
import cz.jzitnik.chronos.entities.Item;
import cz.jzitnik.chronos.interactions.InteractionService;
import cz.jzitnik.chronos.payload.requests.InteractionRequest;
import cz.jzitnik.chronos.payload.responses.InteractionResponse;
import cz.jzitnik.chronos.payload.responses.TakeItemsResponse;
import cz.jzitnik.chronos.payload.responses.UnifiedResponse;
import cz.jzitnik.chronos.repository.CharacterRepository;
import cz.jzitnik.chronos.repository.PlayerRepository;
import cz.jzitnik.chronos.services.ItemService;
import cz.jzitnik.chronos.utils.anotations.CheckUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/game/characters")
public class CharacterController {
    @Autowired
    private InteractionService interactionService;

    @Autowired
    private PlayerRepository playerRepository;

    @Autowired
    private CharacterRepository characterRepository;

    @Autowired
    private ItemService itemService;

    @GetMapping("/interaction")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Interaction, Error>> getInteractionData(@RequestParam String playerKey, @RequestParam Long characterId) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var characterOptional = characterRepository.findById(characterId);

        if (characterOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(UnifiedResponse.failure(null));
        }
        if (!characterOptional.get().getRoom().equals(player.getCurrentRoom())) {
            return ResponseEntity.badRequest().body(UnifiedResponse.failure(null));
        }

        var interaction = characterOptional.get().getInteractionData();

        return ResponseEntity.ok(
                UnifiedResponse.success(interaction)
        );
    }

    @PostMapping("/interact")
    @CheckUser
    public ResponseEntity<UnifiedResponse<InteractionResponse, Error>> interact(@RequestParam String playerKey, @RequestBody InteractionRequest interactionRequest) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var characterOptional = characterRepository.findById(interactionRequest.getCharacterId());

        if (characterOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(UnifiedResponse.failure(null));
        }
        if (!characterOptional.get().getRoom().equals(player.getCurrentRoom())) {
            return ResponseEntity.badRequest().body(UnifiedResponse.failure(null));
        }
        if (characterOptional.get().isInteractedWith()) {
            return ResponseEntity.noContent().build();
        }

        var interaction = characterOptional.get().getInteraction();

        var interactionFunc = interactionService.mapInteraction(interaction);

        var interactionResponse = interactionFunc.apply(player, characterOptional.get(), interactionRequest.getData());

        return ResponseEntity.ok(
                UnifiedResponse.success(interactionResponse)
        );
    }

    @PostMapping("/take_items")
    @CheckUser
    public ResponseEntity<UnifiedResponse<TakeItemsResponse, Error>> takeItems(@RequestParam String playerKey, @RequestParam Long characterId) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var characterOptional = characterRepository.findById(characterId);

        if (characterOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(UnifiedResponse.failure(null));
        }

        var character = characterOptional.get();
        if (!character.getRoom().equals(player.getCurrentRoom())) {
            return ResponseEntity.badRequest().body(UnifiedResponse.failure(null));
        }

        if (character.getPlayersWhoSaw().contains(player)) {
            return ResponseEntity.ok(UnifiedResponse.success(TakeItemsResponse.ALREADY_TAKEN));
        }

        for (Item item : character.getInventory()) {
            var itemClone = new Item(item.getItemType(), player);
            itemService.addItem(player, itemClone);
        }
        player.getSeenCharacters().add(character);

        playerRepository.save(player);

        return ResponseEntity.ok(UnifiedResponse.success(TakeItemsResponse.DONE));
    }
}