package cz.jzitnik.chronos.controllers;

import cz.jzitnik.chronos.entities.Message;
import cz.jzitnik.chronos.payload.requests.MessageRequest;
import cz.jzitnik.chronos.payload.responses.UnifiedResponse;
import cz.jzitnik.chronos.repository.MessageRepository;
import cz.jzitnik.chronos.repository.PlayerRepository;
import cz.jzitnik.chronos.utils.anotations.CheckUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/game/chat")
public class ChatController {
    @Autowired
    private MessageRepository messageRepository;

    @Autowired
    private PlayerRepository playerRepository;

    @GetMapping
    @CheckUser
    public ResponseEntity<UnifiedResponse<List<Message>, Error>> getMessages(@RequestParam String playerKey) {
        var player = playerRepository.findByPlayerKey(playerKey).get();
        var gameId = player.getGame().getId();

        Pageable pageable = PageRequest.of(0, 20);

        var messages = messageRepository.findLastMessagesByGameId(gameId, pageable);

        return ResponseEntity.ok(UnifiedResponse.success(messages));
    }

    @PostMapping
    @CheckUser
    public ResponseEntity<UnifiedResponse<Object, Error>> sendMessage(@RequestParam String playerKey, @RequestBody MessageRequest messageRequest) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        if (messageRequest.getContent().length() > 100 || messageRequest.getContent().isEmpty()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(
                    UnifiedResponse.failure(new Error("Message cannot be longer than 100 characters and cannot be empty!"))
            );
        }

        var message = new Message(player, messageRequest.getContent());

        messageRepository.save(message);

        return ResponseEntity.ok(UnifiedResponse.success(null));
    }
}
