package cz.jzitnik.chronos.controllers;

import cz.jzitnik.chronos.entities.Game;
import cz.jzitnik.chronos.payload.errors.NotFoundError;
import cz.jzitnik.chronos.payload.responses.TestGameKeyResponse;
import cz.jzitnik.chronos.payload.responses.UnifiedResponse;
import cz.jzitnik.chronos.repository.GameRepository;
import cz.jzitnik.chronos.repository.PlayerRepository;
import cz.jzitnik.chronos.services.GameService;
import cz.jzitnik.chronos.utils.anotations.CheckUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/game")
public class GameController {
    @Autowired
    private GameService gameService;

    @Autowired
    private GameRepository gameRepository;

    @Autowired
    private PlayerRepository playerRepository;

    @PostMapping("/new")
    public UnifiedResponse<Game, Error> createGame() {
        var game = gameService.createGame();
        gameRepository.save(game);

        return UnifiedResponse.success(game);
    }

    @PostMapping("/start")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Object, Error>> startGame(@RequestParam String playerKey, @RequestParam String gameKey) {
        var gameOptional = gameRepository.findByGameKey(gameKey);
        var player = playerRepository.findByPlayerKey(playerKey).get();

        if (gameOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(
                    UnifiedResponse.failure(new NotFoundError("Game with key " + gameKey + " was not found!"))
            );
        }

        var game = gameOptional.get();

        if (!game.getAdminPlayer().equals(player)) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(
                    UnifiedResponse.failure(new Error("Not authorized!"))
            );
        }

        game.setStarted(true);

        gameRepository.save(game);

        return ResponseEntity.ok(UnifiedResponse.success(null));
    }

    @GetMapping("/test")
    public ResponseEntity<UnifiedResponse<TestGameKeyResponse, Error>> testGameKey(@RequestParam String gameKey) {
        var gameOptional = gameRepository.findByGameKey(gameKey);

        if (gameOptional.isEmpty()) {
            return ResponseEntity.ok(UnifiedResponse.success(TestGameKeyResponse.INVALID_KEY));
        }

        var game = gameOptional.get();

        if (game.isStarted()) {
            return ResponseEntity.ok(UnifiedResponse.success(TestGameKeyResponse.GAME_STARTED));
        }

        if (game.getPlayers().size() >= 3) {
            return ResponseEntity.ok(UnifiedResponse.success(TestGameKeyResponse.MAXIUM_PLAYERS));
        }

        return ResponseEntity.ok(UnifiedResponse.success(TestGameKeyResponse.WORKING));
    }
}
