package cz.jzitnik.chronos.entities;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonManagedReference;
import jakarta.persistence.*;
import jakarta.validation.constraints.Size;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
@Entity
@NoArgsConstructor
public class Player {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    private String name;

    @OneToMany(mappedBy = "owner", cascade = CascadeType.ALL)
    @JsonManagedReference(value = "player-inventory")
    @Size(max = 3)
    private List<Item> inventory = new ArrayList<>();

    @ManyToOne
    @JsonBackReference(value = "game-players")
    private Game game;

    @ManyToOne
    @JoinColumn(name = "current_room")
    private Room currentRoom;

    @Column(unique = true, nullable = false)
    @JsonIgnore
    private String playerKey;

    private boolean luck = false;

    @OneToMany(mappedBy = "author", cascade = CascadeType.ALL)
    @JsonIgnore
    private List<Message> messages = new ArrayList<>();

    @ManyToMany
    @JoinTable(
            name = "character_player",
            joinColumns = @JoinColumn(name = "player_id"),
            inverseJoinColumns = @JoinColumn(name = "character_id")
    )
    @JsonIgnore
    private List<Character> seenCharacters = new ArrayList<>();

    public Player(String name) {
        this.name = name;
    }

    public void addItem(Item item) {
        inventory.add(item);
    }
}
