package cz.jzitnik.chronos.services;

import cz.jzitnik.chronos.entities.*;
import cz.jzitnik.chronos.repository.GameRepository;
import cz.jzitnik.chronos.repository.PlayerRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.security.SecureRandom;

@Service
public class GameService {
    @Autowired
    private GameRepository gameRepository;

    @Autowired
    private PlayerRepository playerRepository;

    @Autowired
    private InitGameService initGameService;

    private static final int KEY_LENGTH = 10;
    private static final String CHAR_POOL = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    public String generateKey() {
        SecureRandom random = new SecureRandom();
        StringBuilder gameKey = new StringBuilder(KEY_LENGTH);

        for (int i = 0; i < KEY_LENGTH; i++) {
            int index = random.nextInt(CHAR_POOL.length());
            gameKey.append(CHAR_POOL.charAt(index));
        }

        return gameKey.toString();
    }

    public String generateUniqueGameKey() {
        var key = generateKey();

        if (gameRepository.existsByGameKey(key)) {
            return generateUniqueGameKey();
        }

        return key;
    }

    public String generateUniquePlayerKey() {
        var key = generateKey();

        if (playerRepository.existsByPlayerKey(key)) {
            return generateUniquePlayerKey();
        }

        return key;
    }

    public Game createGame() {
        Game newGame = new Game();
        newGame.setGameKey(generateUniqueGameKey());

        var defaultRoom = initGameService.initGame(newGame);

        newGame.setDefaultRoom(defaultRoom);

        return gameRepository.save(newGame);
    }

    public Player addPlayer(Game game, String playerName) {
        var player = new Player(playerName);
        player.setPlayerKey(generateUniquePlayerKey());
        player.setGame(game);
        player.setCurrentRoom(game.getDefaultRoom());
        player.getMessages().add(new Message(player, "", MessageType.JOINED));

        if (game.getPlayers().isEmpty()) {
            game.setAdminPlayer(player);
        }

        game.addPlayer(player);

        gameRepository.save(game);

        return player;
    }


    public boolean isFromThisGame(Game game, Room room) {
        return game.getRooms().stream().anyMatch(rm -> rm.equals(room));
    }
}
