package cz.jzitnik.chronos.utils.anotations;

import cz.jzitnik.chronos.payload.errors.PlayerNotFoundError;
import cz.jzitnik.chronos.payload.responses.UnifiedResponse;
import cz.jzitnik.chronos.repository.PlayerRepository;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

@Aspect
@Component
public class UserCheckAspect {

    @Autowired
    private PlayerRepository playerRepository;

    @Around("@annotation(CheckUser)")
    public Object checkUser(ProceedingJoinPoint joinPoint) throws Throwable {
        String playerKey = (String) joinPoint.getArgs()[0]; // Assuming playerKey is the first argument
        var playerOptional = playerRepository.findByPlayerKey(playerKey);

        if (playerOptional.isEmpty()) {
            return ResponseEntity
                    .status(HttpStatus.NOT_FOUND)
                    .body(UnifiedResponse.failure(new PlayerNotFoundError("Player was not found!")));
        }

        // Proceed with the original method if the player exists
        return joinPoint.proceed();
    }
}
