package cz.jzitnik.api;

import cz.jzitnik.api.requests.InteractionRequest;
import cz.jzitnik.api.requests.MessageRequest;
import cz.jzitnik.api.requests.PlayerNameRequest;
import cz.jzitnik.api.responses.InteractionResponse;
import cz.jzitnik.api.responses.InventoryFullResponse;
import cz.jzitnik.api.responses.UnifiedResponse;
import cz.jzitnik.api.types.*;
import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.GET;
import retrofit2.http.POST;
import retrofit2.http.Query;

import java.util.List;

public interface ApiService {
    @GET("status")
    Call<UnifiedResponse<String, Error>> status();

    @POST("game/new")
    Call<UnifiedResponse<Game, Error>> newGame();

    @POST("game/players/new")
    Call<UnifiedResponse<String, Error>> newPlayer(
            @Query("gameKey") String gameKey,
            @Body PlayerNameRequest requestBody
    );

    @GET("game/players/me")
    Call<UnifiedResponse<Player, Error>> getMe(
            @Query("playerKey") String playerKey
    );

    @GET("game/players/me/inv_full")
    Call<UnifiedResponse<InventoryFullResponse, Error>> inventoryIsFull(
            @Query("playerKey") String playerKey
    );

    @POST("game/players/use_item")
    Call<UnifiedResponse<Object, Error>> useItem(
            @Query("playerKey") String playerKey,
            @Query("itemId") Long itemId
    );

    @POST("game/start")
    Call<UnifiedResponse<Object, Error>> startGame(
            @Query("playerKey") String playerKey,
            @Query("gameKey") String gameKey
    );

    @GET("game/test")
    Call<UnifiedResponse<TestGameKeyResponse, Error>> testGameKey(
            @Query("gameKey") String gameKey
    );

    @GET("game/rooms")
    Call<UnifiedResponse<List<Room>, Error>> getAllRooms(
            @Query("playerKey") String playerKey
    );

    @GET("game/rooms/current_room")
    Call<UnifiedResponse<Room, Error>> getCurrentRoom(
            @Query("playerKey") String playerKey
    );

    @POST("game/rooms/move")
    Call<UnifiedResponse<Object, Error>> moveToRoom(
            @Query("playerKey") String playerKey,
            @Query("roomId") Long roomId
    );

    @POST("game/characters/take_items")
    Call<UnifiedResponse<TakeItemsResponse, Error>> takeItems(
            @Query("playerKey") String playerKey,
            @Query("characterId") Long characterId
    );

    @GET("game/characters/interaction")
    Call<UnifiedResponse<InteractionData, Error>> getInteractionData(
            @Query("playerKey") String playerKey,
            @Query("characterId") Long characterId
    );

    @POST("game/characters/interact")
    Call<UnifiedResponse<InteractionResponse, Error>> interact(
            @Query("playerKey") String playerKey,
            @Body InteractionRequest interactionRequest
    );

    @GET("game/chat")
    Call<UnifiedResponse<List<Message>, Error>> getMessages(
            @Query("playerKey") String playerKey
    );

    @POST("game/chat")
    Call<UnifiedResponse<Object, Error>> sendMessage(
            @Query("playerKey") String playerKey,
            @Body MessageRequest messageRequest
    );

    @GET("game/players/item")
    Call<UnifiedResponse<Item, Error>> getItemInfo(
            @Query("playerKey") String playerKey,
            @Query("itemId") Long itemId
    );

    @GET("game/rooms/room")
    Call<UnifiedResponse<Room, Error>> getRoom(
            @Query("playerKey") String playerKey,
            @Query("roomId") Long roomId
    );
}
