package cz.jzitnik.api.types;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.jzitnik.api.ApiService;
import cz.jzitnik.utils.Cli;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import java.io.IOException;

@Getter
@Setter
@NoArgsConstructor
public class Message {
    private Long id;

    private Player author;

    private String content;

    private MessageType messageType;

    public String getString(ApiService apiService, String playerKey) {
        return switch (messageType) {
            case CUSTOM -> Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + ": " + content;
            case JOINED -> "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " se připojil.";
            case GOT_ITEM -> {
                try {
                    var response = apiService.getItemInfo(playerKey, Long.valueOf(content)).execute();
                    var body = response.body();

                    if (!body.getSuccess()) {
                        yield "Hráč " + Cli.Colors.BLUE +  author.getName() + Cli.Colors.RESET + " získal unknown.";
                    }

                    var item = body.getData().get();

                    yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " získal " + Cli.Colors.BLUE + item + Cli.Colors.RESET + ".";
                } catch (IOException e) {
                    yield "Hráč unknown získal unknown";
                }
            }
            case MOVE_TO_ROOM -> {
                try {
                    var response = apiService.getRoom(playerKey, Long.valueOf(content)).execute();
                    var body = response.body();

                    if (!body.getSuccess()) {
                        yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " vešel do místnosti unknown.";
                    }

                    var room = body.getData().get();

                    yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " vešel do místnosti " + Cli.Colors.BLUE + room + Cli.Colors.RESET + ".";
                } catch (IOException e) {
                    yield "Hráč unknown vešel do místnosti unknown";
                }
            }
            case LOST_ITEM -> {
                ObjectMapper objectMapper = new ObjectMapper();
                try {
                    Item item = objectMapper.readValue(content, Item.class);

                    yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " sice získal " + Cli.Colors.BLUE + item + Cli.Colors.RESET + ", ale měl plný inventář a tím pádem ho " + Cli.Colors.RED + "navždy ztatil" + Cli.Colors.RESET + ".";
                } catch (JsonProcessingException e) {
                    yield "Hráč " + author.getName() + " sice získal item, ale měl plný inventář a tím pádem ho navždy ztratil.";
                }
            }
            case ITEM_USED -> {
                try {
                    var response = apiService.getItemInfo(playerKey, Long.valueOf(content)).execute();

                    var body = response.body();

                    if (!body.getSuccess()) {
                        yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " využil item unknown.";
                    }

                    var item = body.getData().get();

                    yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " využil item " + Cli.Colors.BLUE + item + Cli.Colors.RESET + ".";
                } catch (IOException e) {
                    yield "Hráč " + Cli.Colors.BLUE + author.getName() + Cli.Colors.RESET + " využil item unknown.";
                }
            }
        };
    }
}
