package cz.jzitnik.game;

import cz.jzitnik.api.ApiService;
import cz.jzitnik.api.types.Item;
import cz.jzitnik.api.types.ItemType;
import cz.jzitnik.api.types.Player;
import cz.jzitnik.utils.Cli;
import lombok.AllArgsConstructor;

import java.io.IOException;
import java.util.*;

@AllArgsConstructor
public class Inventory {
    private Player player;
    private ApiService apiService;
    private String playerKey;


    public void display() throws IOException {
        System.out.println();
        Cli.info("Váš inventář:");

        // Create a map to group items by ItemType and count occurrences
        Map<ItemType, Integer> itemCounts = new HashMap<>();
        List<Item> usableItemList = new ArrayList<>();

        for (Item item : player.getInventory()) {
            ItemType itemType = item.getItemType();

            // If the item is usable, add it to the usableItemList
            if (itemType.isUsable()) {
                usableItemList.add(item);
            } else {
                // Otherwise, count it in the itemCounts map
                itemCounts.put(itemType, itemCounts.getOrDefault(itemType, 0) + 1);
            }
        }

        // Display grouped non-usable items
        for (Map.Entry<ItemType, Integer> entry : itemCounts.entrySet()) {
            System.out.println(entry.getValue() + "x " + entry.getKey());
        }

        // Display usable items at the end
        for (Item usableItem : usableItemList) {
            System.out.println("1x " + Cli.Colors.BLUE + usableItem + Cli.Colors.RESET + " (lze využít)");
        }


        if (!usableItemList.isEmpty()) {
            System.out.println("\n\nNyní si můžete vybrat jaký item chcete využít");
            var options = new ArrayList<>(usableItemList.stream().map(Item::toString).toList());
            options.add(Cli.Colors.BLUE + "Zavřít inventář" + Cli.Colors.RESET);

            var selectedIndex = Cli.selectOptionIndex(options);

            if (selectedIndex == options.size() - 1) {
                return;
            }

            var item = usableItemList.get(selectedIndex);

            apiService.useItem(playerKey, item.getId()).execute();

            Cli.info(item + " byl využit!");
        }
    }
}
