package cz.jzitnik.utils;

import cz.jzitnik.api.types.Character;
import cz.jzitnik.api.types.Item;

import java.util.List;
import java.util.Scanner;

public class Cli {
    public static class Colors {
        public static final String RESET = "\033[0m";  // ANSI escape code to reset the terminal

        // Text colors
        public static final String BLACK = "\033[0;30m";
        public static final String RED = "\033[0;31m";
        public static final String GREEN = "\033[0;32m";
        public static final String YELLOW = "\033[0;33m";
        public static final String BLUE = "\033[0;34m";
        public static final String PURPLE = "\033[0;35m";
        public static final String CYAN = "\033[0;36m";
        public static final String WHITE = "\033[0;37m";

        // Bold text colors (in bold)
        public static final String BOLD_BLACK = "\033[1;30m";
        public static final String BOLD_RED = "\033[1;31m";
        public static final String BOLD_GREEN = "\033[1;32m";
        public static final String BOLD_YELLOW = "\033[1;33m";
        public static final String BOLD_BLUE = "\033[1;34m";
        public static final String BOLD_PURPLE = "\033[1;35m";
        public static final String BOLD_CYAN = "\033[1;36m";
        public static final String BOLD_WHITE = "\033[1;37m";

        // Background colors
        public static final String BLACK_BACKGROUND = "\033[40m";
        public static final String RED_BACKGROUND = "\033[41m";
        public static final String GREEN_BACKGROUND = "\033[42m";
        public static final String YELLOW_BACKGROUND = "\033[43m";
        public static final String BLUE_BACKGROUND = "\033[44m";
        public static final String PURPLE_BACKGROUND = "\033[45m";
        public static final String CYAN_BACKGROUND = "\033[46m";
        public static final String WHITE_BACKGROUND = "\033[47m";

        // Bold background colors
        public static final String BOLD_BLACK_BACKGROUND = "\033[1;40m";
        public static final String BOLD_RED_BACKGROUND = "\033[1;41m";
        public static final String BOLD_GREEN_BACKGROUND = "\033[1;42m";
        public static final String BOLD_YELLOW_BACKGROUND = "\033[1;43m";
        public static final String BOLD_BLUE_BACKGROUND = "\033[1;44m";
        public static final String BOLD_PURPLE_BACKGROUND = "\033[1;45m";
        public static final String BOLD_CYAN_BACKGROUND = "\033[1;46m";
        public static final String BOLD_WHITE_BACKGROUND = "\033[1;47m";
    }

    public static void printHeader(String text) {
        int totalWidth = 100;
        int padding = (totalWidth - text.length()) / 2;

        StringBuilder sb = new StringBuilder(totalWidth);
        sb.append("=".repeat(Math.max(0, padding)));
        sb.append(" ");
        sb.append(text);
        sb.append(" ");
        sb.append("=".repeat(Math.max(0, padding)));

        while (sb.length() < totalWidth) {
            sb.insert(0, "=");
        }

        System.out.println();
        System.out.println(sb);
    }

    public static void center(String text) {
        int totalWidth = 100;
        int padding = (totalWidth - text.length()) / 2;

        StringBuilder sb = new StringBuilder(totalWidth);
        sb.append(" ".repeat(Math.max(0, padding)));
        sb.append(text);
        sb.append(" ".repeat(Math.max(0, padding)));

        while (sb.length() < totalWidth) {
            sb.insert(0, " ");
        }

        System.out.println(sb);
    }

    public static String wrapText(String text) {
        return wrapText(text, 0);
    }

    public static String wrapText(String text, int charsRemove) {
        StringBuilder wrappedText = new StringBuilder();
        int line = 0;

        var words = text.split(" ");

        for (var word : words) {
            if ((line + word.length() + 1) > (100 - charsRemove)) {
                wrappedText.append("\n");
                line = 0;
            }
            line += word.length() + 1;
            wrappedText.append(word).append(" ");
        }

        return wrappedText.toString();
    }

    public static void typeText(String text) {
        for (int i = 0; i < text.length(); i++) {
            System.out.print(text.charAt(i));
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        System.out.println();
    }

    public static void gotItems(List<Item> items) {
        if (items.isEmpty()) {
            return;
        }

        var itemsMapped = items.stream().map(Item::toString).toList();
        Cli.info("Dostal jste: " + String.join(", ", itemsMapped));
    }

    public static void type(Character character, String text) {
        Cli.type(Cli.Colors.YELLOW + character.getName() + ": " + Cli.Colors.RESET + text);
    }
    public static void type(String text) {
        Cli.typeText(Cli.wrapText(text));
    }

    public static void info(String text) {
        System.out.println(Colors.BLUE + "Info: " + Colors.RESET + Cli.wrapText(text, 6));
    }
    public static void success(String text) {
        System.out.println(Colors.GREEN + "Úspěch: " + Colors.RESET + Cli.wrapText(text, 8));
    }
    public static void error(String text) {
        System.out.println(Colors.RED + "Chyba: " + Colors.RESET + Cli.wrapText(text, 7));
    }


    /**
     * Displays a menu with the provided options and returns the selected option.
     *
     * @param options Array of options to display to the user.
     * @return The selected option as a String.
     */
    public static int selectOptionIndex(List<String> options) {
        Scanner scanner = new Scanner(System.in);
        int choice;

        System.out.println("Vyberte možnost:");
        for (int i = 0; i < options.size(); i++) {
            System.out.println((i + 1) + ". " + options.get(i));
        }

        while (true) {
            System.out.print("Vložte číslo možnosti: ");
            if (scanner.hasNextInt()) {
                choice = scanner.nextInt();
                if (choice >= 1 && choice <= options.size()) {
                    return choice - 1;
                } else {
                    System.out.println("Neplatní možnost, vyberte číslo mezi 1 a " + options.size());
                }
            } else {
                System.out.println("Neplatný vstup, vložte číslo.");
                scanner.next();
            }
        }
    }
}
